package org.netkernel.doc.book;

import java.util.Map;

import org.w3c.dom.Document;
import org.netkernel.request.IRequest;
import org.netkernel.urii.*;
import org.netkernel.xml.xda.DOMXDA;
import org.netkernel.xml.xda.IXDAIterator;
import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.boot.*;
import org.netkernel.layer0.meta.IEndpointMeta;
import org.netkernel.layer0.meta.IPrototypeMeta;
import org.netkernel.layer0.meta.IRepresentationMeta;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.util.*;

import org.netkernel.module.standard.endpoint.IStandardOverlay;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.module.standard.endpoint.StandardTransportImpl;
import org.netkernel.module.standard.endpoint.TransparentConfiguredOverlayImpl;
import org.netkernel.module.standard.endpoint.TransparentOverlayImpl;

import com.sun.org.apache.bcel.internal.generic.ACONST_NULL;

public class EndpointView extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext context) throws Exception
	{
		IRepDeployedModules mods=context.source("active:moduleList", IRepDeployedModules.class);
		String opd=context.source("arg:operand", String.class);

		Object result=null;
		IMetaRepresentation meta=null;
		String hash=mods.hashForSpaceElement(opd);
		if(hash!=null)
		{	meta=mods.metadataForHash(hash);
		}
		else
		{	meta=mods.metadataForHash(opd);
			hash=opd;
		}
		
		if (meta!=null)
		{
			Document dom=context.transrept(meta, Document.class);
			
			String[] split=hash.split("/");
			String hash2=split[0];
			Object space=mods.componentForHash(hash2);
			String spaceid="";
			String spaceversion="";
			String spaceHash="";
			
			if(space instanceof IModule)
			{	ISpaceWithIdentity[] pub=((IModule)space).getPublicSpaces();
				if(pub.length==1)
				{	spaceid=pub[0].getIdentifier().toString();	
					spaceversion=pub[0].getVersion().toString();
					spaceHash=mods.hashForComponent(pub[0]);
				}
				else
				{	for(int i=0; i<pub.length; i++)
					{	String h3=mods.hashForComponent(pub[i]);
						h3=h3+"/"+split[2];
						if(mods.metadataForHash(h3)!=null)
						{	spaceid=pub[i].getIdentifier().toString();	
							spaceversion=pub[i].getVersion().toString();
							spaceHash=mods.hashForComponent(pub[i]);
							break;
						}
					}
				}
			}
			
			String type = XMLUtils.getText(XMLUtils.getChildElementNamed(dom.getDocumentElement(),"class"));
			if (type.contains("IPrototypeMeta"))
			{	IPrototypeMeta pm = (IPrototypeMeta)meta;
				if (TransparentOverlayImpl.class.isAssignableFrom(pm.getImplementation()) || TransparentConfiguredOverlayImpl.class.isAssignableFrom(pm.getImplementation()))
				{	type="transparent overlay";
				}
				else if (IStandardOverlay.class.isAssignableFrom(pm.getImplementation()))
				{	type="overlay";
				}
				else if (StandardTransportImpl.class.isAssignableFrom(pm.getImplementation()))
				{	type="transport";
				}
				else
				{	type="accessor prototype";
				}
			}
			else if (type.contains("IEndpointMeta"))
			{	IEndpointMeta em = (IEndpointMeta)meta;
				int verbs=em.getSupportedVerbs();
				if (verbs==IRequest.VERB_TRANSREPT)
				{	type="transreptor";
				}
				else
				{	type="accessor";
				}
			}
			else if (type.contains("IRepresentationMeta"))
			{	type="representation";
			}
			
			//append javadoc if necessary
			Document javadoc=null;
			if (meta instanceof IRepresentationMeta)
			{	IRepresentationMeta rm = (IRepresentationMeta)meta;
				String implName=rm.getImplementation().getName();
				
				INKFRequest req2=context.createRequest("active:nkseSearch");
				String search="title:("+implName+") AND category:javadoc";
				req2.addArgumentByValue("operand", search);
				req2.setRepresentationClass(IHDSNode.class);
				IHDSNode sr = (IHDSNode)context.issueRequest(req2);
				IHDSNode firstHit=sr.getFirstNode("/hits/hit");
				if (firstHit!=null)
				{	float score=Float.parseFloat(firstHit.getFirstValue("score").toString());
					if (score>1.0f)
					{	String documentID=null;
						String resolverID=null;
						for(IHDSNode field: firstHit.getNodes("field"))
						{	String name=(String)field.getFirstValue("name");
							if(name.equals("documentID"))
							{	documentID=(String)field.getFirstValue("value");
							}
							else if(name.equals("resolverID"))
							{	resolverID=(String)field.getFirstValue("value");
							}
						}
						if (documentID!=null && resolverID!=null)
						{
							req2=context.createRequest("active:nkseSearchResolver");
							req2.addArgumentByValue("documentID", documentID);
							req2.addArgumentByValue("resolverID", resolverID);
							req2.setRepresentationClass(String.class);
							String path=(String)context.issueRequest(req2);
							//System.out.println("path="+path);
							
							INKFResponseReadOnly resp=context.sourceForResponse("res:"+path);
							req2=context.createRequest("active:tagSoup");
							req2.addArgumentFromResponse("operand", resp);
							req2.setRepresentationClass(Document.class);
							Document d = (Document)context.issueRequest(req2);
							DOMXDA xda = new DOMXDA(d,true);
							xda.removeNS("/html", "");
							int j=path.lastIndexOf("/");
							String javadocBase=path.substring(0,j+1);
							for (IXDAIterator i=xda.iterator("//a"); i.hasNext(); )
							{	i.next();
								try
								{	String href=i.getText("@href", false);
									i.setText("@href", javadocBase+href);
								}
								catch (Exception e)
								{
								}
							}
							
							xda.delete("//table[@class='javadoc-bottom']");
							
							javadoc=xda.getDocument();							
						}
					}
				}
			}
			
			
			
			
					
			INKFRequest req=context.createRequest("active:xslt");
			req.addArgument("operator", "res:/org/netkernel/doc/book/style-meta.xsl");
			req.addArgumentByValue("operand", dom);
			
			//try to find containing book if any
			try
			{	IHDSNode docEntry=context.source("pbv:docentry",IHDSNode.class);
				//System.out.println("docEntry="+docEntry);
				String docId=docEntry.getFirstValue("doc/id").toString();
				//System.out.println("docId="+docId);
				Map documentBookMap = context.source("active:documentBookMap",Map.class);
				String bookId=documentBookMap.get(docId).toString();
				//System.out.println("bookEntry="+bookEntry);
				if (!bookId.equals("book:mod:reference"))
				{	IHDSNode bookEntry = context.source("active:bookEntry+id@"+bookId,IHDSNode.class);
					req.addArgumentByValue("bookEntry", bookEntry);
				}
			}
			catch (Exception e)
			{	
			}
			
			
			req.addArgumentByValue("hash", hash);
			req.addArgumentByValue("shash", spaceHash);
			req.addArgumentByValue("space", spaceid);
			req.addArgumentByValue("version", spaceversion);
			req.addArgumentByValue("type", type);
			req.addArgumentByValue("id", opd);
			if (javadoc!=null)
			{	req.addArgumentByValue("javadoc", javadoc);
			}

			req.setRepresentationClass(String.class);
			result=context.issueRequest(req);
		}
		else
		{	result="<div style='background: red;'>No metadata for  "+opd+"</div>";	
		}

		context.createResponseFrom(result);		
	}
}
